@@ -43,8 +43,9 @@ class ADODB_db2 extends ADOConnection {
 	var $useFetchArray = false; // setting this to true will make array elements in FETCH_ASSOC mode case-sensitive
 								// breaking backward-compat
 	var $_bindInputArray = false;	
+	var $hasGenID = true;
 	var $_genIDSQL = "VALUES NEXTVAL FOR %s";
-	var $_genSeqSQL = "CREATE SEQUENCE %s START WITH 1 NO MAXVALUE NO CYCLE";
+	var $_genSeqSQL = "CREATE SEQUENCE %s START WITH %s NO MAXVALUE NO CYCLE";
 	var $_dropSeqSQL = "DROP SEQUENCE %s";
 	var $_autocommit = true;
 	var $_haserrorfunctions = true;
@@ -224,36 +225,6 @@ class ADODB_db2 extends ADOConnection {
 		}
 	}
 
-	
-	function CreateSequence($seqname='adodbseq',$start=1)
-	{
-		if (empty($this->_genSeqSQL)) return false;
-		$ok = $this->Execute(sprintf($this->_genSeqSQL,$seqname));
-		if (!$ok) return false;
-		return true;
-	}
-	
-	function DropSequence($seqname)
-	{
-		if (empty($this->_dropSeqSQL)) return false;
-		return $this->Execute(sprintf($this->_dropSeqSQL,$seqname));
-	}
-	
-	/*
-		This algorithm is not very efficient, but works even if table locking
-		is not available.
-		
-		Will return false if unable to generate an ID after $MAXLOOPS attempts.
-	*/
-	function GenID($seq='adodbseq',$start=1)
-	{	
-		// if you have to modify the parameter below, your database is overloaded,
-		// or you need to implement generation of id's yourself!
-		$num = $this->GetOne("VALUES NEXTVAL FOR $seq");
-				return $num;
-			}
-
-
 	function ErrorMsg()
 	{
 		if ($this->_haserrorfunctions) {
